﻿using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using StockSharp.Algo.Candles;
using StockSharp.Algo.Storages;
using StockSharp.Algo.Testing;
using StockSharp.Algo.Indicators.Trend;
using StockSharp.BusinessEntities;
using OptimizerStrategy;

namespace Optimizer
{
    class Program
    {
        static void Main()
        {
            //Заголовок приложения
            Console.Title = "Optimizer ver 2.0";

            //Пути к файлам
            const string saveTest = @"L:\DataTesting\";
            const string dataHistoryPath = @"L:\DataHistory";

            //Параметры тестирования
            DateTime startTime = Convert.ToDateTime("01.08.2013");
            DateTime stopTime = Convert.ToDateTime("05.08.2013");
            TimeSpan timeFrame = TimeSpan.FromSeconds(300);
            const string id = "VB12_2008-08_2013@TEST";

            //Выводим на экран
            Console.WriteLine("*Параметры тестирования*");
            Console.WriteLine("Диапазон тестирования с {0}по {1} ", startTime.ToString().Remove(11), stopTime.ToString().Remove(11));
            Console.WriteLine("Таймфрейм: {0} ", timeFrame);
            Console.WriteLine("Инструмент: {0} ", id);
            Console.WriteLine();

            //Задаем переменные параметры стратегии
            const decimal firstParametrStart = 34m;
            const decimal firstParametrEnd = 45m;
            const decimal stepFirstParametr = 1m;

            const decimal secondParametrStart = 5m;
            const decimal secondParametrEnd = 9m;
            const decimal stepSecondParametr = 1m;

            const decimal thirdParametrStart = 1m;
            const decimal thirdParametrEnd = 1m;
            const decimal stepThirdParametr = 1m;

            //Счетчик протестированных вариантов стратегий(массивом предложил Решарпер сделать)
            int[] countReadyStrategy = { 0 };

            //Счетчик готовых стратегий
            int countAllReadyStrategy = 0;

            //Счетчик стратегий завершенных с ошибкой
            int CountErrorStrategy = 0;

            //Счетчик добавленных на тестирование стратегий
            int[] countAddTrader = { 0 };

            #region
            //Определяем длину массива первого параметра
            int lenghtArrayFirstParametr = Convert.ToInt32((firstParametrEnd - firstParametrStart) / stepFirstParametr + 1);

            //Инициализируем массив перврого параметра
            var arrayFirstParametr = new decimal[lenghtArrayFirstParametr];

            //Внутренняя переменная для создания массива
            var tempFirstParametr = firstParametrStart;

            //Создаем массив перврого параметра
            for (int i = 0; i < lenghtArrayFirstParametr; i++)
            {
                //Первое значение равно первому значению диапазона
                if (i == 0) arrayFirstParametr[i] = firstParametrStart;
                else
                {
                    arrayFirstParametr[i] = tempFirstParametr + stepFirstParametr;
                    tempFirstParametr = tempFirstParametr + stepFirstParametr;
                }
            }

            //Определяем длину массива второго параметра
            int lenghtArraySecondParametr = Convert.ToInt32((secondParametrEnd - secondParametrStart) / stepSecondParametr + 1);

            //Инициализируем массив второго параметра
            var arraySecondParametr = new decimal[lenghtArraySecondParametr];

            //Внутренняя переменная для создания массива
            var tempSecondParametr = secondParametrStart;

            //Создаем массив второго параметра
            for (int i = 0; i < lenghtArraySecondParametr; i++)
            {
                //Первое значение равно первому значению диапазона
                if (i == 0) arraySecondParametr[i] = secondParametrStart;
                else
                {
                    arraySecondParametr[i] = tempSecondParametr + stepSecondParametr;
                    tempSecondParametr = tempSecondParametr + stepSecondParametr;
                }
            }


            //Определяем длину массива третьего параметра
            int lenghtArrayThirdParametr = Convert.ToInt32((thirdParametrEnd - thirdParametrStart) / stepThirdParametr + 1);

            //Инициализируем массив третьего параметра
            var arrayThirdParametr = new decimal[lenghtArrayThirdParametr];

            //Внутренняя переменная для создания массива
            var tempThirdParametr = thirdParametrStart;
            //Создаем массив третьего параметра
            for (int i = 0; i < lenghtArrayThirdParametr; i++)
            {
                //Первое значение равно первому значению диапазона
                if (i == 0) arrayThirdParametr[i] = thirdParametrStart;
                else
                {
                    arrayThirdParametr[i] = tempThirdParametr + stepThirdParametr;
                    tempThirdParametr = tempThirdParametr + stepThirdParametr;
                }
            }
            #endregion

            //Общее количество вариаций стратегии
            var countVariation = lenghtArrayFirstParametr * lenghtArraySecondParametr * lenghtArrayThirdParametr;

            //Инициализируем массив хранящий параметры стратегии
            var parameters = new object[countVariation][];

            //Индекс массива
            int countArray = 0;

            //Формируем массив хранящий параметры стратегии
            for (int i = 0; i < lenghtArrayFirstParametr; i++)
            {
                for (int j = 0; j < lenghtArraySecondParametr; j++)
                {
                    for (int k = 0; k < lenghtArrayThirdParametr; k++)
                    {
                        parameters[countArray] = new object[] { arrayFirstParametr[i], arraySecondParametr[j], arrayThirdParametr[k] };
                        countArray++;
                    }
                }
            }

            Console.WriteLine("Диапазон параметра №1 от {0} до {1} с шагом {2} (элементов {3})", firstParametrStart,
                              firstParametrEnd, stepFirstParametr, lenghtArrayFirstParametr);
            Console.WriteLine("Диапазон параметра №2 от {0} до {1} с шагом {2} (элементов {3})", secondParametrStart,
                              secondParametrEnd, stepSecondParametr, lenghtArraySecondParametr);
            Console.WriteLine("Диапазон параметра №3 от {0} до {1} с шагом {2} (элементов {3})", thirdParametrStart,
                              thirdParametrEnd, stepThirdParametr, lenghtArrayThirdParametr);
            Console.WriteLine("Общее количество вариаций стратегии: {0} \n", countVariation);

            //Хранилище, через которое будет производиться доступ к тиковой и котировочной базе
            var storageRegistry = new StorageRegistry();

            //Изменяем путь, используемый по умолчанию
            ((LocalMarketDataDrive)storageRegistry.DefaultDrive).Path = dataHistoryPath;

            //Используем алфавитное хранилище
            ((LocalMarketDataDrive)storageRegistry.DefaultDrive).UseAlphabeticPath = true;

            //Создаем тестовый инструмент, на котором будет производится тестирование
            var security = new Security
                {
                    Id = id,
                    Code = "RTS",
                    Name = "RTS",
                    MinStepSize = 10,
                    MinStepPrice = 2,
                    ExchangeBoard = ExchangeBoard.Forts,
                };

            //Тестовый портфель
            var portfolio = new Portfolio { Name = "test" };

            var series = new CandleSeries(typeof(TimeFrameCandle), security, timeFrame);

            //Загружаем из хранилища свечи, + добавляем один день(косяк - не выводит последний день)
            var candles = storageRegistry.GetCandleStorage(typeof(TimeFrameCandle), security, timeFrame).Load(startTime, stopTime.AddDays(1));

            //Если нет свечек, то выходим
            if (candles.Count <= 0)
            {
                Console.WriteLine("Нет свечек!");
                Console.ReadLine();
                return;
            }

            Console.Write("Запустить тестирование? (Enter)");
            Console.ReadLine();
            Console.WriteLine("Тестирование запущено...\n");

            //Задвем время начала тестирования
            DateTime startEmulationTime = DateTime.Now;

            //Задаем индекс
            int indexArray = 0;

            //Если количество протестированных стратегий меньше количества всех стратегий
            while (countAllReadyStrategy < countVariation)
            {
                //Если количество запущенных трейдеров не равно количеству готовых стратегий(ожидаем завершения тестирования стратегии)
                while (countAddTrader[0] == countReadyStrategy[0])
                {
                    //На всякий случай
                    if (countAllReadyStrategy == countVariation) break;

                    //Решарпер предложил сделать переменные массивом
                    countAddTrader[0] = 0;
                    countReadyStrategy[0] = 0;

                    //Создаем шлюз для эмуляции
                    var trader = new EmulationTrader(new[] { security }, new[] { portfolio }, storageRegistry)
                        {
                            StorageRegistry = storageRegistry,
                            UseCandlesTimeFrame = timeFrame,
                        };

                    //Соединяемся с трейдером и запускаем экспорт
                    trader.Connect();
                    trader.StartExport();

                    //Создаем менеджера свечек
                    var candleManager = new CandleManager(trader);
                    candleManager.Start(series);

                    //Создаем торговую стратегию (передаем стратегии параметры из массива параметров, в int на всякий случай для этой стратегии)
                    var strategy = new SmaStrategy(series, new SimpleMovingAverage { Length = Convert.ToInt32(parameters[indexArray][0]) },
                                                           new SimpleMovingAverage { Length = Convert.ToInt32(parameters[indexArray][1]) })
                        {
                            Volume = 1,
                            Security = security,
                            Portfolio = portfolio,
                            Trader = trader,
                        };

                    //Событие изменения стратегии
                    trader.StateChanged += (oldState, newState) =>
                    {
                        if (trader.State == EmulationStates.Started)
                        {
                            //Запускаем стратегию когда эмулятор запустился
                            strategy.Start();
                            //Добавляем в счетчик, что мы запустили стратегию
                            countAddTrader[0]++;
                        }

                        //Если стратегия остановлена
                        if (trader.State == EmulationStates.Stopped)
                        {
                            //Если стратегия завершилась ошибкой
                            if (trader.IsFinished == false)
                            {
                                //Считаем количество ошибочных стратегий
                                CountErrorStrategy++;
                                
                                //Выводим сообщение об ошибке
                                Console.WriteLine("{0}) Ошибка стратегии. Параметры: ({1}-{2}-{3})",
                                                  CountErrorStrategy, parameters[indexArray][0], parameters[indexArray][1], parameters[indexArray][2]);
                            };

                            int countTrade = 0;
                            decimal price = 0;

                            foreach (MyTrade trade in trader.MyTrades)
                            {
                                price = Math.Round(trade.Trade.Price);
                                countTrade++;
                            }
                            
                            //Считаем процент выполнения
                            int percentReady = 100 * countAllReadyStrategy / countVariation;

                            //Если процент меньше 0, то присваиваем 0, чтобы не было деления на 0
                            if (percentReady < 1) percentReady = 1;
                           
                            //Выводим в заголовок процент выполнения
                            Console.Title = "Optimizer " + percentReady.ToString() + "%";

                            Console.WriteLine("{0}) Параметры стратегии: {1}-{2}-{3} \nКоличество сделок: {4}, Последняя цена: {5}\n",
                                              indexArray+1, parameters[indexArray][0], parameters[indexArray][1], parameters[indexArray][2], countTrade, price);

                            //Увеличиваем счетчик всех готовых стратегий
                            countAllReadyStrategy++;

                            //Добавляем к счетчику индексов
                            indexArray++;

                            //Добавляем к счетчику готовых стратегий
                            countReadyStrategy[0]++;

                            //Чистим память и завершаем процессы стратегии
                            trader.Stop();
                            trader.Dispose();
                            trader = null;
                            strategy.Stop();
                            strategy.Dispose();
                            strategy = null;
                            candleManager.Dispose();
                            candleManager = null;

                            //Если количество всех готовых стратегий ровно количеству всех вариантов, то мы закончили
                            if (countAllReadyStrategy == countVariation)
                            {
                                //Выводим в заголовок процент выполнения
                                Console.Title = "Optimizer 100%";

                                Console.WriteLine("Тестирование закончено за {0} \n", (DateTime.Now - startEmulationTime).ToString().Remove(11));
                                Console.WriteLine("Завершить работу? (Enter)");
                            }

                        }

                    };

                    //Запускаем трейдера
                    trader.Start(startTime, stopTime);
                }
            }            
            Console.ReadLine();
        }
    }
}
